{******************************************************************}
{                                                       	   }
{       Borland Delphi Runtime Library                  	   }
{       Windows Terminal Services interface unit                   }
{ 								   }
{ Portions created by Microsoft are 				   }
{ Copyright (C) 1995-1999 Microsoft Corporation. 		   }
{ All Rights Reserved. 						   }
{ 								   }
{ The original file is: cchannel.h, released June 2000. 	   }
{ The original Pascal code is: CChannel.pas, released Dec 2000     }
{ The initial developers of the Pascal code are Marcel van Brakel, }
{ Nathan Allan and Fabio Bomfim Nunes.        			   }
{ 								   }
{ Portions created by Marcel van Brakel are			   }
{ Copyright (C) 1999, 2000 Marcel van Brakel.            	   }
{ 								   }
{ Contributions:      					           }
{ 								   }
{   Fabio Bomfim Nunes (fbn@alba.com.br)                           }
{   Nathan Allan (nallan@enol.com)                                 }
{ 								   }
{ Obtained through:                               	           }
{ Joint Endeavour of Delphi Innovators (Project JEDI)              }
{								   }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{								   }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                   }
{                                                                  }
{ Software distributed under the License is distributed on an 	   }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License. 			   }
{ 								   }
{******************************************************************}

unit CChannel;

interface

uses
  Windows, PChannel;

//***************************************************************************/
// Name: CHANNEL_INIT_EVENT_FN                                              */
//                                                                          */
// Purpose:                                                                 */
//                                                                          */
// This function is passed to MSTSC on VirtualChannelInit.  It is called by */
// MSTSC to tell the application about interesting events.                  */
//                                                                          */
// Returns:                                                                 */
//                                                                          */
// none                                                                     */
//                                                                          */
// Params:                                                                  */
//                                                                          */
// - pInitHandle - a handle uniquely identifying this connection            */
// - event - the event that has occurred - see CHANNEL_EVENT_XXX below      */
// - pData - data associated with the event - see CHANNEL_EVENT_XXX below   */
// - dataLength - length of the data.                                       */
//                                                                          */
//***************************************************************************/

type
  CHANNEL_INIT_EVENT_FN = procedure (pInitHandle: Pointer; event: UINT;
    pData: Pointer; dataLength: UINT); stdcall;
  {$EXTERNALSYM CHANNEL_INIT_EVENT_FN}
  PCHANNEL_INIT_EVENT_FN = ^CHANNEL_INIT_EVENT_FN;
  {$EXTERNALSYM PCHANNEL_INIT_EVENT_FN}
  TChannelInitEventFn = CHANNEL_INIT_EVENT_FN;
  PChannelInitEventFn = PCHANNEL_INIT_EVENT_FN;

//***************************************************************************/
// Events passed to VirtualChannelInitEvent                                 */
//***************************************************************************/

const

// Client initialized (no data)

  CHANNEL_EVENT_INITIALIZED = 0;
  {$EXTERNALSYM CHANNEL_EVENT_INITIALIZED}

// Connection established (data = name of Server)

  CHANNEL_EVENT_CONNECTED = 1;
  {$EXTERNALSYM CHANNEL_EVENT_CONNECTED}

// Connection established with old Server, so no channel support

  CHANNEL_EVENT_V1_CONNECTED = 2;
  {$EXTERNALSYM CHANNEL_EVENT_V1_CONNECTED}

// Connection ended (no data)

  CHANNEL_EVENT_DISCONNECTED = 3;
  {$EXTERNALSYM CHANNEL_EVENT_DISCONNECTED}

// Client terminated (no data)

  CHANNEL_EVENT_TERMINATED = 4;
  {$EXTERNALSYM CHANNEL_EVENT_TERMINATED}

//***************************************************************************/
// Name: CHANNEL_OPEN_EVENT_FN                                              */
//                                                                          */
// Purpose:                                                                 */
//                                                                          */
// This function is passed to MSTSC on VirtualChannelOpen.  It is called by */
// MSTSC when data is available on the channel.                             */
//                                                                          */
// Returns:                                                                 */
//                                                                          */
// none                                                                     */
//                                                                          */
// Params:                                                                  */
//                                                                          */
// - openHandle - a handle uniquely identifying this channel                */
// - event - event that has occurred - see CHANNEL_EVENT_XXX below          */
// - pData - data received                                                  */
// - dataLength - length of the data                                        */
// - totalLength - total length of data written by the Server               */
// - dataFlags - flags, zero, one or more of:                               */
//   - 0x01 - beginning of data from a single write operation at the Server */
//   - 0x02 - end of data from a single write operation at the Server.      */
//                                                                          */
//***************************************************************************/

type
  CHANNEL_OPEN_EVENT_FN = procedure (openHandle: DWORD; event: UINT; pData: Pointer;
    dataLength, totalLength, dataFlags: Longword); stdcall;
  {$EXTERNALSYM CHANNEL_OPEN_EVENT_FN}
  PCHANNEL_OPEN_EVENT_FN = ^CHANNEL_OPEN_EVENT_FN;
  {$EXTERNALSYM PCHANNEL_OPEN_EVENT_FN}
  TChannelOpenEventFn = CHANNEL_OPEN_EVENT_FN;
  PChannelOpenEventFn = PCHANNEL_OPEN_EVENT_FN;

//***************************************************************************/
// Events passed to VirtualChannelOpenEvent                                 */
//***************************************************************************/

const

// Data received from Server (data = incoming data)

  CHANNEL_EVENT_DATA_RECEIVED = 10;
  {$EXTERNALSYM CHANNEL_EVENT_DATA_RECEIVED}

// VirtualChannelWrite completed (pData - pUserData passed on VirtualChannelWrite)

  CHANNEL_EVENT_WRITE_COMPLETE = 11;
  {$EXTERNALSYM CHANNEL_EVENT_WRITE_COMPLETE}

// VirtualChannelWrite cancelled (pData - pUserData passed on VirtualChannelWrite)

  CHANNEL_EVENT_WRITE_CANCELLED = 12;
  {$EXTERNALSYM CHANNEL_EVENT_WRITE_CANCELLED}

//***************************************************************************/
// Return codes from VirtualChannelXxx functions                            */
//***************************************************************************/

const
  CHANNEL_RC_OK                         = 0;
  {$EXTERNALSYM CHANNEL_RC_OK}
  CHANNEL_RC_ALREADY_INITIALIZED        = 1;
  {$EXTERNALSYM CHANNEL_RC_ALREADY_INITIALIZED}
  CHANNEL_RC_NOT_INITIALIZED            = 2;
  {$EXTERNALSYM CHANNEL_RC_NOT_INITIALIZED}
  CHANNEL_RC_ALREADY_CONNECTED          = 3;
  {$EXTERNALSYM CHANNEL_RC_ALREADY_CONNECTED}
  CHANNEL_RC_NOT_CONNECTED              = 4;
  {$EXTERNALSYM CHANNEL_RC_NOT_CONNECTED}
  CHANNEL_RC_TOO_MANY_CHANNELS          = 5;
  {$EXTERNALSYM CHANNEL_RC_TOO_MANY_CHANNELS}
  CHANNEL_RC_BAD_CHANNEL                = 6;
  {$EXTERNALSYM CHANNEL_RC_BAD_CHANNEL}
  CHANNEL_RC_BAD_CHANNEL_HANDLE         = 7;
  {$EXTERNALSYM CHANNEL_RC_BAD_CHANNEL_HANDLE}
  CHANNEL_RC_NO_BUFFER                  = 8;
  {$EXTERNALSYM CHANNEL_RC_NO_BUFFER}
  CHANNEL_RC_BAD_INIT_HANDLE            = 9;
  {$EXTERNALSYM CHANNEL_RC_BAD_INIT_HANDLE}
  CHANNEL_RC_NOT_OPEN                   = 10;
  {$EXTERNALSYM CHANNEL_RC_NOT_OPEN}
  CHANNEL_RC_BAD_PROC                   = 11;
  {$EXTERNALSYM CHANNEL_RC_BAD_PROC}
  CHANNEL_RC_NO_MEMORY                  = 12;
  {$EXTERNALSYM CHANNEL_RC_NO_MEMORY}
  CHANNEL_RC_UNKNOWN_CHANNEL_NAME       = 13;
  {$EXTERNALSYM CHANNEL_RC_UNKNOWN_CHANNEL_NAME}
  CHANNEL_RC_ALREADY_OPEN               = 14;
  {$EXTERNALSYM CHANNEL_RC_ALREADY_OPEN}
  CHANNEL_RC_NOT_IN_VIRTUALCHANNELENTRY = 15;
  {$EXTERNALSYM CHANNEL_RC_NOT_IN_VIRTUALCHANNELENTRY}
  CHANNEL_RC_NULL_DATA                  = 16;
  {$EXTERNALSYM CHANNEL_RC_NULL_DATA}
  CHANNEL_RC_ZERO_LENGTH                = 17;
  {$EXTERNALSYM CHANNEL_RC_ZERO_LENGTH}

//***************************************************************************/
// Levels of Virtual Channel Support                                        */
//***************************************************************************/

const
  VIRTUAL_CHANNEL_VERSION_WIN2000 = 1;
  {$EXTERNALSYM VIRTUAL_CHANNEL_VERSION_WIN2000}

//***************************************************************************/
// Name: VirtualChannelInit                                                 */
//                                                                          */
// Purpose:                                                                 */
//                                                                          */
// This function is called by the application to register the virtual       */
// channels it wants to have access to.  Note that this does not open the   */
// channels, merely reserves the names for use by this application.  This   */
// function must be called before the Client connects to the Server, hence  */
// it is recommended that it is called from the DLL's initialization        */
// procedure.                                                               */
//                                                                          */
//                                                                          */
// On_return, the channels requested have been registered.  However, other  */
// MSTSC initialization may not yet have completed.  The application        */
// receives a VirtualChannelInitEvent callback with the "Client             */
// initialized" event when all MSTSC initialization is complete.            */
//                                                                          */
// Returns:                                                                 */
//                                                                          */
// CHANNEL_RC_OK                                                            */
// CHANNEL_RC_ALREADY_INITIALIZED                                           */
// CHANNEL_RC_ALREADY_CONNECTED                                             */
// CHANNEL_RC_TOO_MANY_CHANNELS                                             */
// CHANNEL_RC_NOT_IN_VIRTUALCHANNELENTRY                                    */
//                                                                          */
// Parameters                                                               */
//                                                                          */
// - ppInitHandle (returned) - handle to pass to subsequent                 */
//                             VirtualChannelXxx calls                      */
// - pChannel - list of names registered by this application                */
// - channelCount - number of channels registered.                          */
// - versionRequested - level of virtual channel support requested (one of  */
//                      the VIRTUAL_CHANNEL_LEVEL_XXX parameters)           */
// - pChannelInitEventProc - address of VirtualChannelInitEvent procedure   */
//                                                                          */
//***************************************************************************/

type
  VIRTUALCHANNELINIT = function (var ppInitHandle: Pointer;
    pChannel: PChannelDef; channelCount: Integer; versionRequested: ULONG;
    pChannelInitEventProc: PChannelInitEventFn): UINT; stdcall;
  {$EXTERNALSYM VIRTUALCHANNELINIT}
  PVIRTUALCHANNELINIT = ^VIRTUALCHANNELINIT;
  {$EXTERNALSYM PVIRTUALCHANNELINIT}
  TVirtualChannelInit = VIRTUALCHANNELINIT;

//***************************************************************************/
// Name: VirtualChannelOpen                                                 */
//                                                                          */
// Purpose:                                                                 */
//                                                                          */
// This function is called by the application to open a channel.  It cannot */
// be called until a connection is established with a Server.               */
//                                                                          */
// Returns:                                                                 */
//                                                                          */
// CHANNEL_RC_OK                                                            */
// CHANNEL_RC_NOT_INITIALIZED                                               */
// CHANNEL_RC_NOT_CONNECTED                                                 */
// CHANNEL_RC_BAD_CHANNEL_NAME                                              */
// CHANNEL_RC_BAD_INIT_HANDLE                                               */
//                                                                          */
// Params:                                                                  */
//                                                                          */
// - pInitHandle - handle from VirtualChannelInit                           */
//                                                                          */
// - pOpenHandle (returned) - handle to pass to subsequent                  */
//                            VirtualChannelXxx calls                       */
// - pChannelName - name of channel to open                                 */
// - pChannelOpenEventProc - address of VirtualChannelOpenEvent procedure   */
//                                                                          */
//***************************************************************************/

type
  VIRTUALCHANNELOPEN = function (pInitHandle: Pointer; var pOpenHandle: DWORD;
    pChannelName: PChar; pChannelOpenEventProc: PChannelOpenEventFn): UINT; stdcall;
  {$EXTERNALSYM VIRTUALCHANNELOPEN}
  PVIRTUALCHANNELOPEN = ^VIRTUALCHANNELOPEN;
  {$EXTERNALSYM PVIRTUALCHANNELOPEN}
  TVirtualChannelOpen = VIRTUALCHANNELOPEN;

//***************************************************************************/
// Name: VirtualChannelClose                                                */
//                                                                          */
// Purpose:                                                                 */
//                                                                          */
// This function is called to close a previously opened channel.            */
//                                                                          */
// Returns:                                                                 */
//                                                                          */
// CHANNEL_RC_OK                                                            */
// CHANNEL_RC_BAD_CHANNEL_HANDLE                                            */
//                                                                          */
// Params:                                                                  */
//                                                                          */
// - openHandle - handle returned on VirtualChannelOpen                     */
//                                                                          */
//***************************************************************************/

type
  VIRTUALCHANNELCLOSE = function (openHandle: DWORD): UINT; stdcall;
  {$EXTERNALSYM VIRTUALCHANNELCLOSE}
  PVIRTUALCHANNELCLOSE = ^VIRTUALCHANNELCLOSE;
  {$EXTERNALSYM PVIRTUALCHANNELCLOSE}
  TVirtualChannelClose = VIRTUALCHANNELCLOSE;

//***************************************************************************/
// Name: VirtualChannelWrite                                                */
//                                                                          */
// Purpose:                                                                 */
//                                                                          */
// This function is used to send data to the partner app on the Server.     */
//                                                                          */
// VirtualChannelWrite copies the data to one or more network buffers as    */
// necessary.  VirtualChannelWrite ensures that data is sent to the Server  */
// on the right context.  It sends all data on MS TC's Sender thread.       */
//                                                                          */
// VirtualChannelWrite is asynchronous - the VirtualChannelOpenEvent        */
// procedure is called when the write completes.  Until that callback is    */
// made, the caller must not free or reuse the buffer passed on             */
// VirtualChannelWrite.  The caller passes a piece of data (pUserData) to   */
// VirtualChannelWrite, which is returned on the VirtualChannelOpenEvent    */
// callback.  The caller can use this data to identify the write which has  */
// completed.                                                               */
//                                                                          */
//                                                                          */
// Returns:                                                                 */
//                                                                          */
// CHANNEL_RC_OK                                                            */
// CHANNEL_RC_NOT_INITIALIZED                                               */
// CHANNEL_RC_NOT_CONNECTED                                                 */
// CHANNEL_RC_BAD_CHANNEL_HANDLE                                            */
//                                                                          */
// Params:                                                                  */
// - openHandle - handle from VirtualChannelOpen                            */
// - pData - data to write                                                  */
// - datalength - length of data to write                                   */
// - pUserData - user supplied data, returned on VirtualChannelOpenEvent    */
//               when the write completes                                   */
//                                                                          */
//***************************************************************************/

type
  VIRTUALCHANNELWRITE = function (openHandle: DWORD; pData: Pointer;
    dataLength: ULONG; pUserData: Pointer): UINT; stdcall;
  {$EXTERNALSYM VIRTUALCHANNELWRITE}
  PVIRTUALCHANNELWRITE = ^VIRTUALCHANNELWRITE;
  {$EXTERNALSYM PVIRTUALCHANNELWRITE}
  TVirtualChannelWrite = VIRTUALCHANNELWRITE;

//***************************************************************************/
// Structure: CHANNEL_ENTRY_POINTS                                          */
//                                                                          */
// Description: Virtual Channel entry points passed to VirtualChannelEntry  */
//***************************************************************************/

type
  PCHANNEL_ENTRY_POINTS = ^CHANNEL_ENTRY_POINTS;
  {$EXTERNALSYM PCHANNEL_ENTRY_POINTS}
  tagCHANNEL_ENTRY_POINTS = record
    cbSize: DWORD;
    protocolVersion: DWORD;
    pVirtualChannelInit: PVirtualChannelInit;
    pVirtualChannelOpen: PVirtualChannelOpen;
    pVirtualChannelClose: PVirtualChannelClose;
    pVirtualChannelWrite: PVirtualChannelWrite;
  end;
  {$EXTERNALSYM tagCHANNEL_ENTRY_POINTS}
  CHANNEL_ENTRY_POINTS = tagCHANNEL_ENTRY_POINTS;
  {$EXTERNALSYM CHANNEL_ENTRY_POINTS}
  TChannelEntryPoints = CHANNEL_ENTRY_POINTS;
  PChannelEntryPoints = PCHANNEL_ENTRY_POINTS;

//***************************************************************************/
// Name: VirtualChannelEntry                                                */
//                                                                          */
// Purpose:                                                                 */
//                                                                          */
// This function is provided by addin DLLS.  It is called by MSTSC at       */
// initialization to tell the addin DLL the addresses of the                */
// VirtualChannelXxx functions.                                             */
//                                                                          */
// Returns:                                                                 */
//                                                                          */
// TRUE - everything OK                                                     */
// FALSE - error, unload the DLL                                            */
//                                                                          */
// Parameters:                                                              */
//                                                                          */
// - pVirtualChannelInit - pointers to VirtualChannelXxx functions          */
// - pVirtualChannelOpen                                                    */
// - pVirtualChannelClose                                                   */
// - pVirtualChannelWrite                                                   */
//                                                                          */
//***************************************************************************/
type
  VIRTUALCHANNELENTRY = function (pEntryPoints: PChannelEntryPoints): BOOL; stdcall;
  {$EXTERNALSYM VIRTUALCHANNELENTRY}
  PVIRTUALCHANNELENTRY = ^VIRTUALCHANNELENTRY;
  {$EXTERNALSYM PVIRTUALCHANNELENTRY}
  TVirtualChannelEntry = VIRTUALCHANNELENTRY;

implementation

end.
