{******************************************************************}
{                                                       	   }
{       Borland Delphi Runtime Library                  	   }
{       Windows Terminal Services interface unit                   }
{ 								   }
{ Portions created by Microsoft are 				   }
{ Copyright (C) 1995-1999 Microsoft Corporation. 		   }
{ All Rights Reserved. 						   }
{ 								   }
{ The original file is: pchannel.h, released June 2000. 	   }
{ The original Pascal code is: PChannel.pas, released Dec 2000     }
{ The initial developers of the Pascal code are Marcel van Brakel, }
{ Nathan Allan and Fabio Bomfim Nunes.        			   }
{ 								   }
{ Portions created by Marcel van Brakel are			   }
{ Copyright (C) 1999, 2000 Marcel van Brakel.            	   }
{ 								   }
{ Contributions:      					           }
{ 								   }
{   Fabio Bomfim Nunes (fbn@alba.com.br)                           }
{   Nathan Allan (nallan@enol.com)                                 }
{ 								   }
{ Obtained through:                               	           }
{ Joint Endeavour of Delphi Innovators (Project JEDI)              }
{								   }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{								   }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                   }
{                                                                  }
{ Software distributed under the License is distributed on an 	   }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License. 			   }
{ 								   }
{******************************************************************}

unit PChannel;

interface

uses
  Windows;

//***************************************************************************/
// Maximum amount of data that is sent in one operation.  Data larger than  */
// this is segmented into chunks of this size and sent as multiple          */
// operations.                                                              */
//***************************************************************************/

const
  CHANNEL_CHUNK_LENGTH = 1600;
  {$EXTERNALSYM CHANNEL_CHUNK_LENGTH}

//***************************************************************************/
// Header flags (also passed to VirtualChannelOpenEventFn)                  */
//***************************************************************************/

  CHANNEL_FLAG_FIRST  = $01;
  {$EXTERNALSYM CHANNEL_FLAG_FIRST}
  CHANNEL_FLAG_LAST   = $02;
  {$EXTERNALSYM CHANNEL_FLAG_LAST}
  CHANNEL_FLAG_ONLY   = (CHANNEL_FLAG_FIRST or CHANNEL_FLAG_LAST);
  {$EXTERNALSYM CHANNEL_FLAG_ONLY}
  CHANNEL_FLAG_MIDDLE = 0;
  {$EXTERNALSYM CHANNEL_FLAG_MIDDLE}

  CHANNEL_FLAG_FAIL = $100;
  {$EXTERNALSYM CHANNEL_FLAG_FAIL}

//***************************************************************************/
// Header flags (internal protocol use only)                                */
//***************************************************************************/

  CHANNEL_FLAG_SHOW_PROTOCOL = $10;
  {$EXTERNALSYM CHANNEL_FLAG_SHOW_PROTOCOL}
  CHANNEL_FLAG_SUSPEND       = $20;
  {$EXTERNALSYM CHANNEL_FLAG_SUSPEND}
  CHANNEL_FLAG_RESUME        = $40;
  {$EXTERNALSYM CHANNEL_FLAG_RESUME}

//***************************************************************************/
// Virtual Channel options, passed by Client on VirtualChannelOpen          */
//***************************************************************************/

//***************************************************************************/
// Application is initialized.  If this flag is not set, a virtual channel  */
// is not established for this application                                  */
//***************************************************************************/

  CHANNEL_OPTION_INITIALIZED = DWORD($80000000);
  {$EXTERNALSYM CHANNEL_OPTION_INITIALIZED}

//***************************************************************************/
// Encrypt according to RDP data encryption (ie if RDP data is encrypted,   */
// do so for this virtual channel too)                                      */
//***************************************************************************/

  CHANNEL_OPTION_ENCRYPT_RDP = $40000000;
  {$EXTERNALSYM CHANNEL_OPTION_ENCRYPT_RDP}

//***************************************************************************/
// Encrypt Server to Client data (ignored if CHANNEL_OPTION_ENCRYPT_RDP is  */
// set)                                                                     */
//***************************************************************************/

  CHANNEL_OPTION_ENCRYPT_SC = $20000000;
  {$EXTERNALSYM CHANNEL_OPTION_ENCRYPT_SC}

//***************************************************************************/
// Encrypt Client to Server data (ignored if CHANNEL_OPTION_ENCRYPT_RDP is  */
// set)                                                                     */
//***************************************************************************/

  CHANNEL_OPTION_ENCRYPT_CS = $10000000;
  {$EXTERNALSYM CHANNEL_OPTION_ENCRYPT_CS}

//***************************************************************************/
// Send data at high priority (not recommended, as this may impact RDP      */
// performance)                                                             */
//***************************************************************************/

  CHANNEL_OPTION_PRI_HIGH = $08000000;
  {$EXTERNALSYM CHANNEL_OPTION_PRI_HIGH}

//***************************************************************************/
// Send data at medium priority                                             */
//***************************************************************************/

  CHANNEL_OPTION_PRI_MED = $04000000;
  {$EXTERNALSYM CHANNEL_OPTION_PRI_MED}

//***************************************************************************/
// Send data at low priority                                                */
//***************************************************************************/

  CHANNEL_OPTION_PRI_LOW = $02000000;
  {$EXTERNALSYM CHANNEL_OPTION_PRI_LOW}

//***************************************************************************/
// Compress data in this virtual channel if RDP data compression is         */
// configured for this connection                                           */
//***************************************************************************/

  CHANNEL_OPTION_COMPRESS_RDP = $00800000;
  {$EXTERNALSYM CHANNEL_OPTION_COMPRESS_RDP}

//***************************************************************************/
// Compress data in this virtual channel, irrespective of RDP data          */
// compression (ignored if CHANNEL_OPTION_COMPRESS_RDP is set)              */
//***************************************************************************/

  CHANNEL_OPTION_COMPRESS = $00400000;
  {$EXTERNALSYM CHANNEL_OPTION_COMPRESS}

//***************************************************************************/
// Show Server addins the full Virtual Channel protocol.  This option       */
// affects how data passed to VirtualChannelWrite is presented to Server    */
// addins.                                                                  */
//                                                                          */
// - If this option is set, Server addins see the full Virtual Channel      */
// protocol, including CHANNEL_PDU_HEADER (below).                          */
//                                                                          */
// -If this option is not set, Server addins see just the data passed to    */
// VirtualChannelWrite                                                      */
//***************************************************************************/

  CHANNEL_OPTION_SHOW_PROTOCOL = $00200000;
  {$EXTERNALSYM CHANNEL_OPTION_SHOW_PROTOCOL}

//***************************************************************************/
// Maximum number and size of channel names                                 */
//***************************************************************************/

  CHANNEL_MAX_COUNT = 30;
  {$EXTERNALSYM CHANNEL_MAX_COUNT}
  CHANNEL_NAME_LEN  = 7;
  {$EXTERNALSYM CHANNEL_NAME_LEN}

//***************************************************************************/
// Structure: CHANNEL_DEF                                                   */
//                                                                          */
// Description: Client to Server virtual channel information                */
// - name       channel name                                                */
// - options    channel options (a combination of the CHANNEL_OPTION        */
//              constants above)                                            */
//***************************************************************************/

type
  PCHANNEL_DEF = ^CHANNEL_DEF;
  {$EXTERNALSYM PCHANNEL_DEF}
  tagCHANNEL_DEF = record
    name: array [0..CHANNEL_NAME_LEN] of Char;
    options: ULONG;
  end;
  {$EXTERNALSYM tagCHANNEL_DEF}
  CHANNEL_DEF = tagCHANNEL_DEF;
  {$EXTERNALSYM CHANNEL_DEF}
  PPCHANNEL_DEF = ^PCHANNEL_DEF;
  {$EXTERNALSYM PPCHANNEL_DEF}
  TChannelDef = CHANNEL_DEF;
  PChannelDef = PCHANNEL_DEF;

//***************************************************************************/
// Header of Virtual Channel PDUs                                           */
//***************************************************************************/

//***************************************************************************/
// Structure: CHANNEL_PDU_HEADER                                            */
//                                                                          */
// Description: Header sent on Virtual Channel PDUs                         */
//***************************************************************************/

type
  PCHANNEL_PDU_HEADER = ^CHANNEL_PDU_HEADER;
  {$EXTERNALSYM PCHANNEL_PDU_HEADER}
  tagCHANNEL_PDU_HEADER = record
    length: Longword; // Length of data including header
    flags: Longword;  // CHANNEL_FLAG_xxx flags
  end;
  {$EXTERNALSYM tagCHANNEL_PDU_HEADER}
  CHANNEL_PDU_HEADER = tagCHANNEL_PDU_HEADER;
  {$EXTERNALSYM CHANNEL_PDU_HEADER}
  TChannelPduHeader = CHANNEL_PDU_HEADER;
  PChannelPduHeader = PCHANNEL_PDU_HEADER;

//***************************************************************************/

const
  CHANNEL_PDU_LENGTH = CHANNEL_CHUNK_LENGTH + SizeOf(CHANNEL_PDU_HEADER);
  {$EXTERNALSYM CHANNEL_PDU_LENGTH}

implementation

end.
