{******************************************************************}
{                                                       	   }
{       Borland Delphi Runtime Library                  	   }
{       Citrix Terminal Services interface unit                    }
{ 								   }
{ Portions created by Microsoft are 				   }
{ Copyright (C) 1995-1999 Microsoft Corporation. 		   }
{ All Rights Reserved. 						   }
{ 								   }
{ The original file is: wfapi.h, released June 2000. 	           }
{ The original Pascal code is: WfApi.pas, released December 2000   }
{ The initial developers of the Pascal code are Marcel van Brakel, }
{ Nathan Allan and Fabio Bomfim Nunes.        			   }
{ 								   }
{ Portions created by Marcel van Brakel are			   }
{ Copyright (C) 1999, 2000 Marcel van Brakel.            	   }
{ 								   }
{ Contributions:      					           }
{ 								   }
{   Fabio Bomfim Nunes (fbn@alba.com.br)                           }
{   Nathan Allan (nallan@enol.com)                                 }
{ 								   }
{ Obtained through:                               	           }
{ Joint Endeavour of Delphi Innovators (Project JEDI)              }
{								   }
{ You may retrieve the latest version of this file at the Project  }
{ JEDI home page, located at http://delphi-jedi.org                }
{								   }
{ The contents of this file are used with permission, subject to   }
{ the Mozilla Public License Version 1.1 (the "License"); you may  }
{ not use this file except in compliance with the License. You may }
{ obtain a copy of the License at                                  }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                   }
{                                                                  }
{ Software distributed under the License is distributed on an 	   }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   }
{ implied. See the License for the specific language governing     }
{ rights and limitations under the License. 			   }
{ 								   }
{******************************************************************}

unit WFApi;

interface

uses
  Windows;

//   Citrix server public APIs
//
//  Copyright 1995-1998, Citrix Systems Inc.

//==============================================================================
// Defines
//==============================================================================

//
//  Specifies the current server
//

const
  WF_CURRENT_SERVER        = THandle(0);
  {$EXTERNALSYM WF_CURRENT_SERVER}
  WF_CURRENT_SERVER_HANDLE = THandle(0);
  {$EXTERNALSYM WF_CURRENT_SERVER_HANDLE}
  WF_CURRENT_SERVER_NAME   = '';
  {$EXTERNALSYM WF_CURRENT_SERVER_NAME}

//
//  Specifies the current session (SessionId)
//

  WF_CURRENT_SESSION = DWORD(-1);
  {$EXTERNALSYM WF_CURRENT_SESSION}

//
//  Possible pResponse values from WFSendMessage()
//

  IDTIMEOUT = 32000;
  {$EXTERNALSYM IDTIMEOUT}
  IDASYNC   = 32001;
  {$EXTERNALSYM IDASYNC}

//
//  Shutdown flags
//

  WF_WSD_LOGOFF     = $00000001;   // log off all users except
  {$EXTERNALSYM WF_WSD_LOGOFF}     // current user; deletes
                                   // WinStations (a reboot is
                                   // required to recreate the
                                   // WinStations)
  WF_WSD_SHUTDOWN   = $00000002;   // shutdown system
  {$EXTERNALSYM WF_WSD_SHUTDOWN}
  WF_WSD_REBOOT     = $00000004;   // shutdown and reboot
  {$EXTERNALSYM WF_WSD_REBOOT}
  WF_WSD_POWEROFF   = $00000008;   // shutdown and power off (on
  {$EXTERNALSYM WF_WSD_POWEROFF}   // machines that support power
                                   // off through software)
  WF_WSD_FASTREBOOT = $00000010;   // reboot without logging users
  {$EXTERNALSYM WF_WSD_FASTREBOOT} // off or shutting down

//==============================================================================
// WF_CONNECTSTATE_CLASS - Session connect state
//==============================================================================

type
  _WF_CONNECTSTATE_CLASS = (
    WFActive,              // User logged on to WinStation
    WFConnected,           // WinStation connected to client
    WFConnectQuery,        // In the process of connecting to client
    WFShadow,              // Shadowing another WinStation
    WFDisconnected,        // WinStation logged on without client
    WFIdle,                // Waiting for client to connect
    WFListen,              // WinStation is listening for connection
    WFReset,               // WinStation is being reset
    WFDown,                // WinStation is down due to error
    WFInit);               // WinStation in initialization
  {$EXTERNALSYM _WF_CONNECTSTATE_CLASS}
  WF_CONNECTSTATE_CLASS = _WF_CONNECTSTATE_CLASS;
  {$EXTERNALSYM WF_CONNECTSTATE_CLASS}
  TWfConnectStateClass = WF_CONNECTSTATE_CLASS;

//==============================================================================
// WF_SERVER_INFO - returned by WFEnumerateServers (version 1)
//==============================================================================

//
//  WFEnumerateServers() returns two variables: pServerInfo and Count.
//  The latter is the number of WF_SERVER_INFO structures contained in
//  the former.  In order to read each server, iterate i from 0 to
//  Count-1 and reference the server name as
//  pServerInfo[i].pServerName; for example:
//
//  for ( i=0; i < Count; i++ ) {
//      _tprintf( TEXT("%s "), pServerInfo[i].pServerName );
//  }
//
//  The memory returned looks like the following.  P is a pServerInfo
//  pointer, and D is the string data for that pServerInfo:
//
//  P1 P2 P3 P4 ... Pn D1 D2 D3 D4 ... Dn
//
//  This makes it easier to iterate the servers, using code similar to
//  the above.
//

type
  PWF_SERVER_INFOW = ^WF_SERVER_INFOW;
  {$EXTERNALSYM PWF_SERVER_INFOW}
  _WF_SERVER_INFOW = record
    pServerName: PWideChar; // server name
  end;
  {$EXTERNALSYM _WF_SERVER_INFOW}
  WF_SERVER_INFOW = _WF_SERVER_INFOW;
  {$EXTERNALSYM WF_SERVER_INFOW}
  TWfServerInfoW = WF_SERVER_INFOW;
  PWfServerInfoW = PWF_SERVER_INFOW;

  PWF_SERVER_INFOA = ^WF_SERVER_INFOA;
  {$EXTERNALSYM PWF_SERVER_INFOA}
  _WF_SERVER_INFOA = record
    pServerName: PAnsiChar; // server name
  end;
  {$EXTERNALSYM _WF_SERVER_INFOA}
  WF_SERVER_INFOA = _WF_SERVER_INFOA;
  {$EXTERNALSYM WF_SERVER_INFOA}
  TWfServerInfoA = WF_SERVER_INFOA;
  PWfServerInfoA = PWF_SERVER_INFOA;

{$IFDEF UNICODE}
  WF_SERVER_INFO  = WF_SERVER_INFOW;
  {$EXTERNALSYM WF_SERVER_INFO}
  PWF_SERVER_INFO = PWF_SERVER_INFOW;
  {$EXTERNALSYM PWF_SERVER_INFO}
  TWfServerInfo = TWfServerInfoW;
  PWfServerInfo = PWfServerInfoW;
{$ELSE}
  WF_SERVER_INFO  = WF_SERVER_INFOA;
  {$EXTERNALSYM WF_SERVER_INFO}
  PWF_SERVER_INFO = PWF_SERVER_INFOA;
  {$EXTERNALSYM PWF_SERVER_INFO}
  TWfServerInfo = TWfServerInfoA;
  PWfServerInfo = PWfServerInfoA;
{$ENDIF}

//==============================================================================
// WF_SESSION_INFO - returned by WFEnumerateSessions (version 1)
//==============================================================================

//
//  WFEnumerateSessions() returns data in a similar format to the above
//  WFEnumerateServers().  It returns two variables: pSessionInfo and
//  Count.  The latter is the number of WF_SESSION_INFO structures
//  contained in the former.  Iteration is similar, except that there
//  are three parts to each entry, so it would look like this:
//
//  for ( i=0; i < Count; i++ ) {
//      _tprintf( TEXT("%-5u  %-20s  %u\n"),
//                pSessionInfo[i].SessionId,
//                pSessionInfo[i].pWinStationName,
//                pSessionInfo[i].State );
//  }
//
//  The memory returned is also segmented as the above, with all the
//  structures allocated at the start and the string data at the end.
//  We'll use S for the SessionId, P for the pWinStationName pointer
//  and D for the string data, and C for the connect State:
//
//  S1 P1 C1 S2 P2 C2 S3 P3 C3 S4 P4 C4 ... Sn Pn Cn D1 D2 D3 D4 ... Dn
//
//  As above, this makes it easier to iterate the sessions.
//

type
  PWF_SESSION_INFOW = ^WF_SESSION_INFOW;
  {$EXTERNALSYM PWF_SESSION_INFOW}
  _WF_SESSION_INFOW = record
    SessionId: DWORD;             // session id
    pWinStationName: PWideChar;      // name of WinStation this session is connected to
    State: WF_CONNECTSTATE_CLASS; // connection state (see enum)
  end;
  {$EXTERNALSYM _WF_SESSION_INFOW}
  WF_SESSION_INFOW = _WF_SESSION_INFOW;
  {$EXTERNALSYM WF_SESSION_INFOW}
  TWfSessionInfoW = WF_SESSION_INFOW;
  PWfSessionInfoW = PWF_SESSION_INFOW;

  PWF_SESSION_INFOA = ^WF_SESSION_INFOA;
  {$EXTERNALSYM PWF_SESSION_INFOA}
  _WF_SESSION_INFOA = record
    SessionId: DWORD;             // session id
    pWinStationName: PAnsiChar;       // name of WinStation this session is connected to
    State: TWfConnectStateClass; // connection state (see enum)
  end;
  {$EXTERNALSYM _WF_SESSION_INFOA}
  WF_SESSION_INFOA = _WF_SESSION_INFOA;
  {$EXTERNALSYM WF_SESSION_INFOA}
  TWfSessionInfoA = WF_SESSION_INFOA;
  PWfSessionInfoA = PWF_SESSION_INFOA;

{$IFDEF UNICODE}
  WF_SESSION_INFO  = WF_SESSION_INFOW;
  {$EXTERNALSYM WF_SESSION_INFO}
  PWF_SESSION_INFO = PWF_SESSION_INFOW;
  {$EXTERNALSYM PWF_SESSION_INFO}
  TWfSessionInfo = TWfSessionInfoW;
  PWfSessionInfo = PWfSessionInfoW;
{$ELSE}
  WF_SESSION_INFO  = WF_SESSION_INFOA;
  {$EXTERNALSYM WF_SESSION_INFO}
  PWF_SESSION_INFO = PWF_SESSION_INFOA;
  {$EXTERNALSYM PWF_SESSION_INFO}
  TWfSessionInfo = TWfSessionInfoA;
  PWfSessionInfo = PWfSessionInfoA;
{$ENDIF}

//==============================================================================
// WF_PROCESS_INFO - returned by WFEnumerateProcesses (version 1)
//==============================================================================

//
//  WFEnumerateProcesses() also returns data similar to
//  WFEnumerateServers().  It returns two variables: pProcessInfo and
//  Count.  The latter is the number of WF_PROCESS_INFO structures
//  contained in the former.  Iteration is similar, except that there
//  are four parts to each entry, so it would look like this:
//
//  for ( i=0; i < Count; i++ ) {
//      GetUserNameFromSid( pProcessInfo[i].pUserSid, UserName,
//                          sizeof(UserName) );
//      _tprintf( TEXT("%-5u  %-20s  %-5u  %s\n"),
//              pProcessInfo[i].SessionId,
//              UserName,
//              pProcessInfo[i].ProcessId,
//              pProcessInfo[i].pProcessName );
//  }
//
//  The memory returned is also segmented as the above, with all the
//  structures allocated at the start and the string data at the end.
//  We'll use S for the SessionId, R for the ProcessId, P for the
//  pProcessName pointer and D for the string data, and U for pUserSid:
//
//  S1 R1 P1 U1 S2 R2 P2 U2 S3 R3 P3 U3 ... Sn Rn Pn Un D1 D2 D3 ... Dn
//
//  As above, this makes it easier to iterate the processes.
//

type
  PWF_PROCESS_INFOW = ^WF_PROCESS_INFOW;
  {$EXTERNALSYM PWF_PROCESS_INFOW}
  _WF_PROCESS_INFOW = record
    SessionId: DWORD;     // session id
    ProcessId: DWORD;     // process id
    pProcessName: PWideChar; // name of process
    pUserSid: PSID;       // user's SID
  end;
  {$EXTERNALSYM _WF_PROCESS_INFOW}
  WF_PROCESS_INFOW = _WF_PROCESS_INFOW;
  {$EXTERNALSYM WF_PROCESS_INFOW}
  TWfProcessInfoW = WF_PROCESS_INFOW;
  PWfProcessInfoW = PWF_PROCESS_INFOW;

  PWF_PROCESS_INFOA = ^WF_PROCESS_INFOA;
  {$EXTERNALSYM PWF_PROCESS_INFOA}
  _WF_PROCESS_INFOA = record
    SessionId: DWORD;    // session id
    ProcessId: DWORD;    // process id
    pProcessName: PAnsiChar; // name of process
    pUserSid: PSID;      // user's SID
  end;
  {$EXTERNALSYM _WF_PROCESS_INFOA}
  WF_PROCESS_INFOA = _WF_PROCESS_INFOA;
  {$EXTERNALSYM WF_PROCESS_INFOA}
  TWfProcessInfoA = WF_PROCESS_INFOA;
  PWfProcessInfoA = PWF_PROCESS_INFOA;

{$IFDEF UNICODE}
  WF_PROCESS_INFO  = WF_PROCESS_INFOW;
  {$EXTERNALSYM WF_PROCESS_INFO}
  PWF_PROCESS_INFO = PWF_PROCESS_INFOW;
  {$EXTERNALSYM PWF_PROCESS_INFO}
  TWfProcessInfo = TWfProcessInfoW;
  PWfProcessInfo = PWfProcessInfoW;
{$ELSE}
  WF_PROCESS_INFO  = WF_PROCESS_INFOA;
  {$EXTERNALSYM WF_PROCESS_INFO}
  PWF_PROCESS_INFO = PWF_PROCESS_INFOA;
  {$EXTERNALSYM PWF_PROCESS_INFO}
  TWfProcessInfo = TWfProcessInfoA;
  PWfProcessInfo = PWfProcessInfoA;
{$ENDIF}

//==============================================================================
// WF_INFO_CLASS - WFQuerySessionInformation
//(See additional typedefs for more info on structures)
//==============================================================================

type
  _WF_INFO_CLASS = (
    WFVersion,             // OSVERSIONINFO
    WFInitialProgram,
    WFWorkingDirectory,
    WFOEMId,
    WFSessionId,
    WFUserName,
    WFWinStationName,
    WFDomainName,
    WFConnectState,
    WFClientBuildNumber,
    WFClientName,
    WFClientDirectory,
    WFClientProductId,
    WFClientHardwareId,
    WFClientAddress,
    WFClientDisplay,
    WFClientCache,
    WFClientDrives,
    WFICABufferLength,
    WFLicenseEnabler,
    RESERVED2,
    WFApplicationName);
  {$EXTERNALSYM _WF_INFO_CLASS}
  WF_INFO_CLASS = _WF_INFO_CLASS;
  {$EXTERNALSYM WF_INFO_CLASS}
  TWfInfoClass = WF_INFO_CLASS;

//==============================================================================
// WFQuerySessionInformation - (WFClientProductId)
//
// NOTE: the following are Citrix-defined client product IDs.  OEM
//       clients will have different, OEM-specific values.
//==============================================================================

const
  CLIENTID_CITRIX_DOS       = $0001; // citrix dos client
  {$EXTERNALSYM CLIENTID_CITRIX_DOS}
  CLIENTID_CITRIX_TEXT_TERM = $0004; // citrix text terminals
  {$EXTERNALSYM CLIENTID_CITRIX_TEXT_TERM}
  CLIENTID_CITRIX_DIRECTICA = $0007; // citrix DirectICA clients
  {$EXTERNALSYM CLIENTID_CITRIX_DIRECTICA}

//==============================================================================
// WFQuerySessionInformation - (WFClientAddress)
//==============================================================================

type
  PWF_CLIENT_ADDRESS = ^WF_CLIENT_ADDRESS;
  {$EXTERNALSYM PWF_CLIENT_ADDRESS}
  _WF_CLIENT_ADDRESS = record
    AddressFamily: DWORD;           // AF_INET, AF_IPX, AF_NETBIOS, AF_UNSPEC
    Address: array [0..19] of Byte; // client network address
  end;
  {$EXTERNALSYM _WF_CLIENT_ADDRESS}
  WF_CLIENT_ADDRESS = _WF_CLIENT_ADDRESS;
  {$EXTERNALSYM WF_CLIENT_ADDRESS}
  TWfClientAddress = WF_CLIENT_ADDRESS;
  PWfClientAddress = PWF_CLIENT_ADDRESS;

//==============================================================================
// WFQuerySessionInformation - (WFClientDisplay)
//==============================================================================

type
  PWF_CLIENT_DISPLAY = ^WF_CLIENT_DISPLAY;
  {$EXTERNALSYM PWF_CLIENT_DISPLAY}
  _WF_CLIENT_DISPLAY = record
    HorizontalResolution: DWORD; // horizontal dimensions, in pixels
    VerticalResolution: DWORD; // vertical dimensions, in pixels
    ColorDepth: DWORD; // 1=16, 2=256, 4=64K, 8=16M
  end;
  {$EXTERNALSYM _WF_CLIENT_DISPLAY}
  WF_CLIENT_DISPLAY = _WF_CLIENT_DISPLAY;
  {$EXTERNALSYM WF_CLIENT_DISPLAY}
  TWfClientDisplay = WF_CLIENT_DISPLAY;
  PWfClientDisplay = PWF_CLIENT_DISPLAY;

//==============================================================================
// WFQuerySessionInformation - (WFClientCache)
//==============================================================================

type
  PWF_CLIENT_CACHE = ^WF_CLIENT_CACHE;
  {$EXTERNALSYM PWF_CLIENT_CACHE}
  _WF_CLIENT_CACHE = record
    CacheTiny: DWORD;          // size of cache for small objects (32 bytes)
    CacheLowMem: DWORD;        // size of cache (low memory for DOS;
                               // virtual memory for Win16/Win32)
    CacheXms: DWORD;           // size of cache in XMS memory (DOS only)
    CacheDisk: DWORD;          // size of cache in disk (DOS only)
    DimCacheSize: DWORD;       // persistent cache size
    DimBitmapMin: DWORD;       // minimum bitmap size for persistent cache
    DimSignatureLevel: DWORD;  // version of persistent caching
    DimFilesysOverhead: DWORD; // The amount of overhead that the file
                               // system on the client requires per bitmap
  end;
  {$EXTERNALSYM _WF_CLIENT_CACHE}
  WF_CLIENT_CACHE = _WF_CLIENT_CACHE;
  {$EXTERNALSYM WF_CLIENT_CACHE}
  TWfClientCache = WF_CLIENT_CACHE;
  PWfClientCache = PWF_CLIENT_CACHE;

//==============================================================================
// WFQuerySessionInformation - (WFClientDrives)
//==============================================================================

type
  PWF_DRIVES = ^WF_DRIVES;
  {$EXTERNALSYM PWF_DRIVES}
  _WF_DRIVES = record
    DriveLetter: DWORD; // in ASCII format, i.e. 'C'
    Flags: DWORD; // see below for flag values
  end;
  {$EXTERNALSYM _WF_DRIVES}
  WF_DRIVES = _WF_DRIVES;
  {$EXTERNALSYM WF_DRIVES}
  TWfDrives = WF_DRIVES;
  PWfDrives = PWF_DRIVES;

//
//  Client drive mapping flags (Flags)
//

const
  WF_DRIVE_REMOVEABLE = $0001;
  {$EXTERNALSYM WF_DRIVE_REMOVEABLE}
  WF_DRIVE_FIXED      = $0002;
  {$EXTERNALSYM WF_DRIVE_FIXED}
  WF_DRIVE_REMOTE     = $0004;
  {$EXTERNALSYM WF_DRIVE_REMOTE}
  WF_DRIVE_CDROM      = $0008;
  {$EXTERNALSYM WF_DRIVE_CDROM}

type
  PWF_CLIENT_DRIVES = ^WF_CLIENT_DRIVES;
  {$EXTERNALSYM PWF_CLIENT_DRIVES}
  _WF_CLIENT_DRIVES = record
    fAutoClientDrives: DWORD;           // are client drives auto-connected?
    Count: DWORD;                       // number of actual client drives
    Drives: array [0..25] of TWfDrives; // client drives (room for 26 letters)
  end;
  {$EXTERNALSYM _WF_CLIENT_DRIVES}
  WF_CLIENT_DRIVES = _WF_CLIENT_DRIVES;
  {$EXTERNALSYM WF_CLIENT_DRIVES}
  TWfClientDrives = WF_CLIENT_DRIVES;
  PWfClientDrives = PWF_CLIENT_DRIVES;

//==============================================================================
// WF_CONFIG_CLASS - WFQueryUserConfig/WFSetUserConfig
//==============================================================================

type
  _WF_CONFIG_CLASS = (
    WFUserConfigInitialProgram,          // string returned/expected
    WFUserConfigWorkingDirectory,        // string returned/expected
    WFUserConfigfInheritInitialProgram); // DWORD returned/expected
  {$EXTERNALSYM _WF_CONFIG_CLASS}
  WF_CONFIG_CLASS = _WF_CONFIG_CLASS;
  {$EXTERNALSYM WF_CONFIG_CLASS}
  TWfConfigClass = WF_CONFIG_CLASS;

//==============================================================================
// WF_EVENT - Event flags for WFWaitSystemEvent
//==============================================================================

const
  WF_EVENT_NONE        = $00000000; // return no event
  {$EXTERNALSYM WF_EVENT_NONE}
  WF_EVENT_CREATE      = $00000001; // new WinStation created
  {$EXTERNALSYM WF_EVENT_CREATE}
  WF_EVENT_DELETE      = $00000002; // existing WinStation deleted
  {$EXTERNALSYM WF_EVENT_DELETE}
  WF_EVENT_RENAME      = $00000004; // existing WinStation renamed
  {$EXTERNALSYM WF_EVENT_RENAME}
  WF_EVENT_CONNECT     = $00000008; // WinStation connect to client
  {$EXTERNALSYM WF_EVENT_CONNECT}
  WF_EVENT_DISCONNECT  = $00000010; // WinStation logged on without client
  {$EXTERNALSYM WF_EVENT_DISCONNECT}
  WF_EVENT_LOGON       = $00000020; // user logged on to existing WinStation
  {$EXTERNALSYM WF_EVENT_LOGON}
  WF_EVENT_LOGOFF      = $00000040; // user logged off from existing WinStation
  {$EXTERNALSYM WF_EVENT_LOGOFF}
  WF_EVENT_STATECHANGE = $00000080; // WinStation state change
  {$EXTERNALSYM WF_EVENT_STATECHANGE}
  WF_EVENT_LICENSE     = $00000100; // license state change
  {$EXTERNALSYM WF_EVENT_LICENSE}
  WF_EVENT_ALL         = $7fffffff; // wait for all event types
  {$EXTERNALSYM WF_EVENT_ALL}
  WF_EVENT_FLUSH       = DWORD($80000000); // unblock all waiters
  {$EXTERNALSYM WF_EVENT_FLUSH}

//==============================================================================
// WF_VIRTUAL_CLASS - WFVirtualChannelQuery
//==============================================================================

type
  _WF_VIRTUAL_CLASS = (WFVirtualClientData); // Virtual channel client module data (C2H data)
  {$EXTERNALSYM _WF_VIRTUAL_CLASS}
  WF_VIRTUAL_CLASS = _WF_VIRTUAL_CLASS;
  {$EXTERNALSYM WF_VIRTUAL_CLASS}
  TWfVirtualClass = WF_VIRTUAL_CLASS;

//==============================================================================
// Citrix server public APIs
//==============================================================================

function WFEnumerateServersA(pDomainName: PAnsiChar; Reserved: DWORD; Version: DWORD;
  var ppServerInfo: PWfServerInfoA; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateServersA}
function WFEnumerateServersW(pDomainName: PWideChar; Reserved: DWORD; Version: DWORD;
  var ppServerInfo: PWfServerInfoW; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateServersW}

{$IFDEF UNICODE}
function WFEnumerateServers(pDomainName: PWideChar; Reserved: DWORD; Version: DWORD;
  var ppServerInfo: PWfServerInfoW; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateServers}
{$ELSE}
function WFEnumerateServers(pDomainName: PAnsiChar; Reserved: DWORD; Version: DWORD;
  var ppServerInfo: PWfServerInfoA; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateServers}
{$ENDIF}

//------------------------------------------------------------------------------

function WFOpenServerA(pServerName: PAnsiChar): THandle; stdcall;
{$EXTERNALSYM WFOpenServerA}
function WFOpenServerW(pServerName: PWideChar): THandle; stdcall;
{$EXTERNALSYM WFOpenServerW}

{$IFDEF UNICODE}
function WFOpenServer(pServerName: PWideChar): THandle; stdcall;
{$EXTERNALSYM WFOpenServer}
{$ELSE}
function WFOpenServer(pServerName: PAnsiChar): THandle; stdcall;
{$EXTERNALSYM WFOpenServer}
{$ENDIF}

//------------------------------------------------------------------------------
procedure WFCloseServer(hServer: THandle); stdcall;
{$EXTERNALSYM WFCloseServer}

//------------------------------------------------------------------------------

function WFEnumerateSessionsA(hServer: THandle; Reserved: DWORD; Version: DWORD;
  var ppSessionInfo: PWfSessionInfoA; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateSessionsA}
function WFEnumerateSessionsW(hServer: THandle; Reserved: DWORD; Version: DWORD;
  var ppSessionInfo: PWfSessionInfoW; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateSessionsW}

{$IFDEF UNICODE}
function WFEnumerateSessions(hServer: THandle; Reserved: DWORD; Version: DWORD;
  var ppSessionInfo: PWfSessionInfoW; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateSessions}
{$ELSE}
function WFEnumerateSessions(hServer: THandle; Reserved: DWORD; Version: DWORD;
  var ppSessionInfo: PWfSessionInfoA; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateSessions}
{$ENDIF}

//------------------------------------------------------------------------------

function WFEnumerateProcessesA(hServer: THandle; Reserved: DWORD; Version: DWORD;
  var ppProcessInfo: PWfProcessInfoA; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateProcessesA}
function WFEnumerateProcessesW(hServer: THandle; Reserved: DWORD; Version: DWORD;
  var ppProcessInfo: PWfProcessInfoW; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateProcessesW}

{$IFDEF UNICODE}
function WFEnumerateProcesses(hServer: THandle; Reserved: DWORD; Version: DWORD;
  var ppProcessInfo: PWfProcessInfoW; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateProcesses}
{$ELSE}
function WFEnumerateProcesses(hServer: THandle; Reserved: DWORD; Version: DWORD;
  var ppProcessInfo: PWfProcessInfoA; var pCount: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFEnumerateProcesses}
{$ENDIF}

//------------------------------------------------------------------------------

function WFTerminateProcess(hServer: THandle; ProcessId, ExitCode: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFTerminateProcess}

//------------------------------------------------------------------------------

function WFQuerySessionInformationA(hServer: THandle; SessionId: DWORD;
  WFInfoClass: TWfInfoClass; var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFQuerySessionInformationA}
function WFQuerySessionInformationW(hServer: THandle; SessionId: DWORD;
  WFInfoClass: TWfInfoClass; var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFQuerySessionInformationW}

{$IFDEF UNICODE}
function WFQuerySessionInformation(hServer: THandle; SessionId: DWORD;
  WFInfoClass: TWfInfoClass; var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFQuerySessionInformation}
{$ELSE}
function WFQuerySessionInformation(hServer: THandle; SessionId: DWORD;
  WFInfoClass: TWfInfoClass; var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFQuerySessionInformation}
{$ENDIF}

//------------------------------------------------------------------------------

function WFQueryUserConfigA(pServerName: PAnsiChar; pUserName: PAnsiChar;
  WFConfigClass: TWfConfigClass; var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFQueryUserConfigA}
function WFQueryUserConfigW(pServerName: PWideChar; pUserName: PWideChar;
  WFConfigClass: TWfConfigClass; var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFQueryUserConfigW}

{$IFDEF UNICODE}
function WFQueryUserConfig(pServerName: PWideChar; pUserName: PWideChar;
  WFConfigClass: TWfConfigClass; var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFQueryUserConfig}
{$ELSE}
function WFQueryUserConfig(pServerName: PAnsiChar; pUserName: PAnsiChar;
  WFConfigClass: TWfConfigClass; var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFQueryUserConfig}
{$ENDIF}

//------------------------------------------------------------------------------

function WFSetUserConfigA(pServerName: PAnsiChar; pUserName: PAnsiChar;
  WFConfigClass: TWfConfigClass; pBuffer: PAnsiChar; DataLength: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFSetUserConfigA}
function WFSetUserConfigW(pServerName: PWideChar; pUserName: PWideChar;
  WFConfigClass: TWfConfigClass; pBuffer: PWideChar; DataLength: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFSetUserConfigW}

{$IFDEF UNICODE}
function WFSetUserConfig(pServerName: PWideChar; pUserName: PWideChar;
  WFConfigClass: TWfConfigClass; pBuffer: PWideChar; DataLength: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFSetUserConfig}
{$ELSE}
function WFSetUserConfig(pServerName: PAnsiChar; pUserName: PAnsiChar;
  WFConfigClass: TWfConfigClass; pBuffer: PAnsiChar; DataLength: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFSetUserConfig}
{$ENDIF}

//------------------------------------------------------------------------------

function WFSendMessageA(hServer: THandle; SessionId: DWORD; pTitle: PAnsiChar;
  TitleLength: DWORD; pMessage: PAnsiChar; MessageLength: DWORD; Style: DWORD;
  Timeout: DWORD; var pResponse: DWORD; bWait: BOOL): BOOL; stdcall;
{$EXTERNALSYM WFSendMessageA}
function WFSendMessageW(hServer: THandle; SessionId: DWORD; pTitle: PWideChar;
  TitleLength: DWORD; pMessage: PWideChar; MessageLength: DWORD; Style: DWORD;
  Timeout: DWORD; var pResponse: DWORD; bWait: BOOL): BOOL; stdcall;
{$EXTERNALSYM WFSendMessageW}

{$IFDEF UNICODE}
function WFSendMessage(hServer: THandle; SessionId: DWORD; pTitle: PWideChar;
  TitleLength: DWORD; pMessage: PWideChar; MessageLength: DWORD; Style: DWORD;
  Timeout: DWORD; var pResponse: DWORD; bWait: BOOL): BOOL; stdcall;
{$EXTERNALSYM WFSendMessage}
{$ELSE}
function WFSendMessage(hServer: THandle; SessionId: DWORD; pTitle: PAnsiChar;
  TitleLength: DWORD; pMessage: PAnsiChar; MessageLength: DWORD; Style: DWORD;
  Timeout: DWORD; var pResponse: DWORD; bWait: BOOL): BOOL; stdcall;
{$EXTERNALSYM WFSendMessage}
{$ENDIF}

//------------------------------------------------------------------------------

function WFDisconnectSession(hServer: THandle; SessionId: DWORD; bWait: BOOL): BOOL; stdcall;
{$EXTERNALSYM WFDisconnectSession}

//------------------------------------------------------------------------------

function WFLogoffSession(hServer: THandle; SessionId: DWORD; bWait: BOOL): BOOL; stdcall;
{$EXTERNALSYM WFLogoffSession}

//------------------------------------------------------------------------------

function WFShutdownSystem(hServer: THandle; ShutdownFlag: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFShutdownSystem}

//------------------------------------------------------------------------------

function WFWaitSystemEvent(hServer: THandle; EventMask: DWORD; var pEventFlags: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFWaitSystemEvent}

//------------------------------------------------------------------------------

function WFVirtualChannelOpen(hServer: THandle; SessionId: DWORD; pVirtualName: PAnsiChar): THandle; stdcall;
{$EXTERNALSYM WFVirtualChannelOpen}

function WFVirtualChannelClose(hChannelHandle: THandle): BOOL; stdcall;
{$EXTERNALSYM WFVirtualChannelClose}

function WFVirtualChannelRead(hChannelHandle: THandle; TimeOut: ULONG; Buffer: PChar;
  BufferSize: ULONG; var pBytesRead: ULONG): BOOL; stdcall;
{$EXTERNALSYM WFVirtualChannelRead}

function WFVirtualChannelWrite(hChannelHandle: THandle; Buffer: PChar; Length: ULONG;
  var pBytesWritten: ULONG): BOOL; stdcall;
{$EXTERNALSYM WFVirtualChannelWrite}

function WFVirtualChannelPurgeInput(hChannelHandle: THandle): BOOL; stdcall;
{$EXTERNALSYM WFVirtualChannelPurgeInput}

function WFVirtualChannelPurgeOutput(hChannelHandle: THandle): BOOL; stdcall;
{$EXTERNALSYM WFVirtualChannelPurgeOutput}

function WFVirtualChannelQuery(hChannelHandle: THandle; VirtualClass: TWfVirtualClass;
  var ppBuffer: Pointer; var pBytesReturned: DWORD): BOOL; stdcall;
{$EXTERNALSYM WFVirtualChannelQuery}

//------------------------------------------------------------------------------

procedure WFFreeMemory(pMemory: Pointer); stdcall;
{$EXTERNALSYM WFFreeMemory}

function WFShadowSessionA(pServerName: PAnsiChar; SessionID, HotKey, HKModifier: ULONG): BOOL; stdcall;
{$EXTERNALSYM WFShadowSessionA}
function WFShadowSessionW(pServerName: PWideChar; SessionID, HotKey, HKModifier: ULONG): BOOL; stdcall;
{$EXTERNALSYM WFShadowSessionW}

{$IFDEF UNICODE}
function WFShadowSession(pServerName: PWideChar; SessionID, HotKey, HKModifier: ULONG): BOOL; stdcall;
{$EXTERNALSYM WFShadowSession}
{$ELSE}
function WFShadowSession(pServerName: PAnsiChar; SessionID, HotKey, HKModifier: ULONG): BOOL; stdcall;
{$EXTERNALSYM WFShadowSession}
{$ENDIF}

implementation

const
  wfapilib = 'wfapi.dll';

function WFEnumerateServersA; external wfapilib name 'WFEnumerateServersA';
function WFEnumerateServersW; external wfapilib name 'WFEnumerateServersW';
{$IFDEF UNICODE}
function WFEnumerateServers; external wfapilib name 'WFEnumerateServersW';
{$ELSE}
function WFEnumerateServers; external wfapilib name 'WFEnumerateServersA';
{$ENDIF}
function WFOpenServerA; external wfapilib name 'WFOpenServerA';
function WFOpenServerW; external wfapilib name 'WFOpenServerW';
{$IFDEF UNICODE}
function WFOpenServer; external wfapilib name 'WFOpenServerW';
{$ELSE}
function WFOpenServer; external wfapilib name 'WFOpenServerA';
{$ENDIF}
procedure WFCloseServer; external wfapilib name 'WFCloseServer';
function WFEnumerateSessionsA; external wfapilib name 'WFEnumerateSessionsA';
function WFEnumerateSessionsW; external wfapilib name 'WFEnumerateSessionsW';
{$IFDEF UNICODE}
function WFEnumerateSessions; external wfapilib name 'WFEnumerateSessionsW';
{$ELSE}
function WFEnumerateSessions; external wfapilib name 'WFEnumerateSessionsA';
{$ENDIF}
function WFEnumerateProcessesA; external wfapilib name 'WFEnumerateProcessesA';
function WFEnumerateProcessesW; external wfapilib name 'WFEnumerateProcessesW';
{$IFDEF UNICODE}
function WFEnumerateProcesses; external wfapilib name 'WFEnumerateProcessesW';
{$ELSE}
function WFEnumerateProcesses; external wfapilib name 'WFEnumerateProcessesA';
{$ENDIF}
function WFTerminateProcess; external wfapilib name 'WFTerminateProcess';
function WFQuerySessionInformationA; external wfapilib name 'WFQuerySessionInformationA';
function WFQuerySessionInformationW; external wfapilib name 'WFQuerySessionInformationW';
{$IFDEF UNICODE}
function WFQuerySessionInformation; external wfapilib name 'WFQuerySessionInformationW';
{$ELSE}
function WFQuerySessionInformation; external wfapilib name 'WFQuerySessionInformationA';
{$ENDIF}
function WFQueryUserConfigA; external wfapilib name 'WFQueryUserConfigA';
function WFQueryUserConfigW; external wfapilib name 'WFQueryUserConfigW';
{$IFDEF UNICODE}
function WFQueryUserConfig; external wfapilib name 'WFQueryUserConfigW';
{$ELSE}
function WFQueryUserConfig; external wfapilib name 'WFQueryUserConfigA';
{$ENDIF}
function WFSetUserConfigA; external wfapilib name 'WFSetUserConfigA';
function WFSetUserConfigW; external wfapilib name 'WFSetUserConfigW';
{$IFDEF UNICODE}
function WFSetUserConfig; external wfapilib name 'WFSetUserConfigW';
{$ELSE}
function WFSetUserConfig; external wfapilib name 'WFSetUserConfigA';
{$ENDIF}
function WFSendMessageA; external wfapilib name 'WFSendMessageA';
function WFSendMessageW; external wfapilib name 'WFSendMessageW';
{$IFDEF UNICODE}
function WFSendMessage; external wfapilib name 'WFSendMessageW';
{$ELSE}
function WFSendMessage; external wfapilib name 'WFSendMessageA';
{$ENDIF}
function WFDisconnectSession; external wfapilib name 'WFDisconnectSession';
function WFLogoffSession; external wfapilib name 'WFLogoffSession';
function WFShutdownSystem; external wfapilib name 'WFShutdownSystem';
function WFWaitSystemEvent; external wfapilib name 'WFWaitSystemEvent';
function WFVirtualChannelOpen; external wfapilib name 'WFVirtualChannelOpen';
function WFVirtualChannelClose; external wfapilib name 'WFVirtualChannelClose';
function WFVirtualChannelRead; external wfapilib name 'WFVirtualChannelRead';
function WFVirtualChannelWrite; external wfapilib name 'WFVirtualChannelWrite';
function WFVirtualChannelPurgeInput; external wfapilib name 'WFVirtualChannelPurgeInput';
function WFVirtualChannelPurgeOutput; external wfapilib name 'WFVirtualChannelPurgeOutput';
function WFVirtualChannelQuery; external wfapilib name 'WFVirtualChannelQuery';
procedure WFFreeMemory; external wfapilib name 'WFFreeMemory';
function WFShadowSessionA; external wfapilib name 'WFShadowSessionA';
function WFShadowSessionW; external wfapilib name 'WFShadowSessionW';
{$IFDEF UNICODE}
function WFShadowSession; external wfapilib name 'WFShadowSessionW';
{$ELSE}
function WFShadowSession; external wfapilib name 'WFShadowSessionA';
{$ENDIF}

end.
